import { S as SvelteComponent, i as init, s as safe_not_equal, p as create_slot, e as element, b as attr, d as toggle_class, f as insert, u as update_slot_base, q as get_all_dirty_from_scope, r as get_slot_changes, j as transition_in, k as transition_out, n as detach } from './index.b2f2b9fa.js';
import { c as create_classes } from './styles.c72835fa.js';

/* src/components/Row/Row.svelte generated by Svelte v3.49.0 */

function create_fragment(ctx) {
	let div;
	let div_class_value;
	let current;
	const default_slot_template = /*#slots*/ ctx[4].default;
	const default_slot = create_slot(default_slot_template, ctx, /*$$scope*/ ctx[3], null);

	return {
		c() {
			div = element("div");
			if (default_slot) default_slot.c();
			attr(div, "class", div_class_value = "flex gr-gap gr-form-gap row gap-4 w-full flex-wrap " + create_classes(/*style*/ ctx[0]));
			attr(div, "id", /*elem_id*/ ctx[1]);
			toggle_class(div, "unequal-height", /*style*/ ctx[0].equal_height === false);
			toggle_class(div, "!hidden", !/*visible*/ ctx[2]);
		},
		m(target, anchor) {
			insert(target, div, anchor);

			if (default_slot) {
				default_slot.m(div, null);
			}

			current = true;
		},
		p(ctx, [dirty]) {
			if (default_slot) {
				if (default_slot.p && (!current || dirty & /*$$scope*/ 8)) {
					update_slot_base(
						default_slot,
						default_slot_template,
						ctx,
						/*$$scope*/ ctx[3],
						!current
						? get_all_dirty_from_scope(/*$$scope*/ ctx[3])
						: get_slot_changes(default_slot_template, /*$$scope*/ ctx[3], dirty, null),
						null
					);
				}
			}

			if (!current || dirty & /*style*/ 1 && div_class_value !== (div_class_value = "flex gr-gap gr-form-gap row gap-4 w-full flex-wrap " + create_classes(/*style*/ ctx[0]))) {
				attr(div, "class", div_class_value);
			}

			if (!current || dirty & /*elem_id*/ 2) {
				attr(div, "id", /*elem_id*/ ctx[1]);
			}

			if (dirty & /*style, style*/ 1) {
				toggle_class(div, "unequal-height", /*style*/ ctx[0].equal_height === false);
			}

			if (dirty & /*style, visible*/ 5) {
				toggle_class(div, "!hidden", !/*visible*/ ctx[2]);
			}
		},
		i(local) {
			if (current) return;
			transition_in(default_slot, local);
			current = true;
		},
		o(local) {
			transition_out(default_slot, local);
			current = false;
		},
		d(detaching) {
			if (detaching) detach(div);
			if (default_slot) default_slot.d(detaching);
		}
	};
}

function instance($$self, $$props, $$invalidate) {
	let { $$slots: slots = {}, $$scope } = $$props;
	let { style = {} } = $$props;
	let { elem_id } = $$props;
	let { visible = true } = $$props;

	$$self.$$set = $$props => {
		if ('style' in $$props) $$invalidate(0, style = $$props.style);
		if ('elem_id' in $$props) $$invalidate(1, elem_id = $$props.elem_id);
		if ('visible' in $$props) $$invalidate(2, visible = $$props.visible);
		if ('$$scope' in $$props) $$invalidate(3, $$scope = $$props.$$scope);
	};

	return [style, elem_id, visible, $$scope, slots];
}

class Row extends SvelteComponent {
	constructor(options) {
		super();
		init(this, options, instance, create_fragment, safe_not_equal, { style: 0, elem_id: 1, visible: 2 });
	}
}

var Row$1 = Row;

const modes = ["static"];

export { Row$1 as Component, modes };
