span.svelte-1cgkd5k{text-shadow:0 0 8px rgba(0, 0, 0, 0.5)}progress.svelte-1cgkd5k::-webkit-progress-bar{border-radius:2px;background-color:rgba(255, 255, 255, 0.2);overflow:hidden}progress.svelte-1cgkd5k::-webkit-progress-value{background-color:rgba(255, 255, 255, 0.9)}.mirror.svelte-1cgkd5k{transform:scaleX(-1)}.wrap.svelte-1cgkd5k{width:calc(100% - 0.375rem * 2)}