div.svelte-10ogue4>*:not(.absolute){border-radius:0px !important
}div.svelte-10ogue4>*:first-child{border-top-left-radius:0.5rem !important;border-top-right-radius:0.5rem !important
}div.svelte-10ogue4>*:last-child{border-bottom-left-radius:0.5rem !important;border-bottom-right-radius:0.5rem !important
}div.svelte-10ogue4>* + *:not(.absolute){border-top-width:0px !important
}